--[[	A special thank you to DarlosD9 and Darkstorm Technologies for this list of key numbers.
	A - 1		Numpad 0 - 37
	B - 2		Numpad 1 - 38
	C - 3		Numpad 2 - 39
	D - 4		Numpad 3 - 40
	E - 5		Numpad 4 - 41
	F - 6		Numpad 5 - 42
	G - 7		Numpad 6 - 43
	H - 8		Numpad 7 - 44
	I - 9		Numpad 8 - 45
	J - 10		Numpad 9 - 46
	K - 11		- - 61
	L - 12		= - 62
	M - 13		{ - 65
	N - 14		} - 66
	O - 15		 " - 69
	P - 16		\ - 70
	Q - 17		, - 72
	R - 18		. - 73
	S - 19		ins - 76
	T - 20		del - 77
	U - 21		home - 78
	V - 22		end - 79
	W - 23		page up - 80
	X - 24		page down - 81
	Y - 25		left arrow - 82
	Z - 26		right arrow - 83
	0 - 27		up arrow - 84
	1 - 28		down arrow - 85
	2 - 29		numpad / - 86
	3 - 30		numpad * - 87
	4 - 31		numpad - - 88
	5 - 32		numpad + - 89
	6 - 33		numpad . - 90
	7 - 34		; - 105
	8 - 35		
	9 - 36
]]

function UAVSecondaryKey()
	return (UInputMan:KeyHeld(24) or UInputMan:KeyHeld(26))	-- X or Z
end

function UAVInitialize(self)
	self.MGTimer = Timer()
	self.DropTimer = Timer()
	self.ThrottleTimer = Timer()
	
	self.throttle = 6200
	self.maxThrot = 8700
	self.minThrot = 500
	self.ammo = 300		-- Machine Guns
	self.liftMultiplier = 14.3
	self.liftMaximum = 13200
	self.stallSpeed = 9
	self.Vel.X = self.stallSpeed * 1.2
	self.Vel.Y = 0
	self.RotAngle = -0.05
	self.angle = 0
	self.speed = 0
	self.occlusion = 0
	self.payload = 0
	self.bomb = ""
	self.playerCtrl = false
	
	-- Minimap
	self.GroundDots = {}
	self.DotPos = {}
	self.dotRes = 4		-- n pixel spacing in the minimap
	self.xDots = math.ceil(SceneMan.SceneWidth / 350)		-- one dot every n pixels in the map
	self.yDots = math.ceil(SceneMan.SceneHeight / 350)
	
	local xgap = math.floor(SceneMan.SceneWidth / self.xDots)
	local Radar = CreateMOPixel("Ground", "UAV.rte")
	Radar.Pos.Y = 0
	
	self.xDots = self.xDots * self.dotRes
	self.yDots = self.yDots * self.dotRes
	self.xDiv  = SceneMan.SceneWidth / self.xDots
	self.yDiv  = SceneMan.SceneHeight / self.yDots
	
	for i = 1, self.xDots/self.dotRes do
		Radar.Pos.X = math.floor(xgap*(i-1) + xgap/2)
		table.insert(self.DotPos, i, Vector(math.ceil(Radar.Pos.X/self.xDiv), math.ceil(Radar:GetAltitude(0,7)/self.yDiv+self.yDots+20)))
	end
end
--[[
function UAVDestroy(self)
	self.playerCtrl = false
	SceneMan:SetScreenOcclusion(Vector(0,0), self.Team)
	for i = 1, #self.GroundDots do
		self.GroundDots[i].ToDelete = true
		self.GroundDots[i] = nil
	end
end

function UAVOffset(Origin, ang, dist)
	local Tmp = Vector(0, 0)
	Tmp:SetMagnitude(dist)
	Tmp:RadRotate(ang)
	Tmp = Tmp + Origin
	return Tmp
end

function UAVAddIcon(self, Icon, dist)
	Icon.Vel = self.Vel
	Icon.Pos = self.Pos + self:RotateOffset(Vector(0, -math.abs(Icon.SpriteOffset.Y) - math.abs(self.SpriteOffset.Y) - dist))
	MovableMan:AddParticle(Icon)
end

function UAVMove(self)
	-- Fake sharp aim		TODO: Move this
	if self:IsPlayerControlled() then
		local Acc = UAVOffset(Vector(0,0), self.angle + math.pi, FrameMan.ResX / self.occlusion * math.max(self.speed, self.stallSpeed))
		SceneMan:SetScreenOcclusion(Acc, self.Team)
	end
	
	-- Throttle
	if self:GetController():IsState(Controller.MOVE_UP) and self.ThrottleTimer:IsPastSimMS(100) and self.throttle < self.maxThrot then
		self.ThrottleTimer:Reset()
		self.throttle = self.throttle + self.minThrot
	elseif self:GetController():IsState(Controller.MOVE_DOWN) and self.ThrottleTimer:IsPastSimMS(100) and self.throttle > self.minThrot then
		self.ThrottleTimer:Reset()
		self.throttle = self.throttle - self.minThrot
	end
	
	-- Turn slower at high and low speeds
	if self.speed > self.stallSpeed then
		if self:GetController():IsState(Controller.MOVE_LEFT) and self.AngularVel < 1.5 then
			self.AngularVel = self.AngularVel + math.max(0.24 - math.abs(24 - self.speed) / 85, 0.02)
		elseif self:GetController():IsState(Controller.MOVE_RIGHT) and self.AngularVel > -1.5 then
			self.AngularVel = self.AngularVel - math.max(0.24 - math.abs(24 - self.speed) / 85, 0.02)
		else
			self.AngularVel = self.AngularVel * 0.93
		end
	end
	
	-- Direct the velocity towards the view angle
	if self.speed > self.stallSpeed then
		local Momentum = Vector(0, 0)
		local mag = self.Vel.Magnitude
		Momentum:SetMagnitude(mag * 0.037)
		Momentum:RadRotate(self.angle)
		Momentum = Momentum + self.Vel
		Momentum:SetMagnitude(mag)
		self.Vel.X, self.Vel.Y = Momentum.X, Momentum.Y
	else	-- stall
		if self.angle < 1.570796 and self.angle > -1.047197 then	-- pi/3 = 1.047197
			self.AngularVel = self.AngularVel - 0.012
		elseif self.angle > 1.570796 or self.angle < -2.094395 then	-- 2*pi/3 = 2.094395
			self.AngularVel = self.AngularVel + 0.012
		end
	end
	
	-- Add engine acceleration
	local attack = math.abs(self.Vel.AbsRadAngle - self.angle)
	local Acc = Vector(0, 0)
	if self.speed < 26 then
		Acc:SetMagnitude(self.throttle)
	else
		Acc:SetMagnitude(self.throttle * 0.87)
	end
	
	Acc:RadRotate(self.angle)
	
	-- Add lift
	if self.speed > 5 then
		local Lft = Vector(0, 0)
		Lft:SetMagnitude(math.min(self.speed^2 * math.max(1.2-attack, 0) * self.liftMultiplier, self.liftMaximum))
		if self.HFlipped == false then
			Lft:RadRotate(self.angle + 1.570796 + 0.05)
		else
			Lft:RadRotate(self.angle - 1.570796 - 0.05)
		end

		Acc = Acc + Lft
	end
	
	-- Add extra drag when angle of attack is high
	if attack > 0.2 and self.Vel.Magnitude > 11 then
		local Drg = Vector(0, 0)
		if attack > 1.570796 then
			attack = math.pi - attack
		end

		Drg:SetMagnitude((self.Vel.Magnitude - self.speed)^1.6 * (attack+0.1) * 500 + 1500)
		Drg:RadRotate(self.Vel.AbsRadAngle + math.pi)
		Acc = Acc + Drg
	end
	
	self:AddAbsForce(Acc, self.Pos)
	
    if self.Health < 1 and self.Vel.Magnitude < 20 then
        self:GibThis()
    end
end

function UAVMap(self)
	if SceneMan.SceneWrapsX then
		local startDot = math.ceil((self.xDots/self.dotRes)*(self.Pos.X/SceneMan.SceneWidth))
		if self.Vel.X > 0 then	-- moving to the right
			for i = startDot, #self.DotPos do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "UAV.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X + (i-startDot)*self.dotRes
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
			
			local lastDotX = self.GroundDots[#self.DotPos].Pos.X
			for i = 1, startDot - 1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "UAV.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = lastDotX + i*self.dotRes
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
		else	-- moving to the left
			local pos = 0
			for i = startDot, 1, -1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "UAV.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X - pos
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				pos = pos + self.dotRes
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
			
			for i = #self.DotPos, startDot + 1, -1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "UAV.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X - pos
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				pos = pos + self.dotRes
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
		end
		
		-- Update UAV dot
		local Px = CreateMOPixel("PlayerPos", "UAV.rte")
		Px.Pos = Vector(self.Pos.X, self.Pos.Y + self.Pos.Y/self.yDiv + self.yDots + 20)
		Px.Vel = self.Vel
		MovableMan:AddParticle(Px)
	else
		for i = 1, #self.DotPos do
			if self.GroundDots[i] == nil or not MovableMan:ValidMO(self.GroundDots[i]) then
				self.GroundDots[i] = CreateMOPixel("Ground", "UAV.rte")
				MovableMan:AddParticle(self.GroundDots[i])
			end
			
			self.GroundDots[i].Pos = self.DotPos[i] + Vector(self.Pos.X - self.xDots/2, self.Pos.Y)
			self.GroundDots[i].Vel = self.Vel
		end
		
		-- Update UAV dot
		local Px = CreateMOPixel("PlayerPos", "UAV.rte")
		Px.Pos = self.Pos + Vector(self.Pos.X/self.xDiv - self.xDots/2, self.Pos.Y/self.yDiv + self.yDots + 20)
		Px.Vel = self.Vel
		MovableMan:AddParticle(Px)
	end
end
		
function UAVFireMG(self)
	if self.MGTimer:IsPastSimMS(85) then	-- 2x705 rpm
		self.MGTimer:Reset()
		
		local bulletType = "UAV MG"
		if self.ammo % 3 == 0 then
			bulletType = "UAV MG Tracer"
		end
		
		local Emitter = CreateAEmitter(bulletType)		
		Emitter.Vel = self.Vel
		Emitter.Pos = self.Pos + self:RotateOffset(Vector(math.abs(self.SpriteOffset.X) + 16, 12))
		Emitter.RotAngle = self.angle
		MovableMan:AddParticle(Emitter)
		
		local Casing = CreateMOSParticle("MG Casing")
		Casing.Pos = self.Pos + self:RotateOffset(Vector(13, math.abs(self.SpriteOffset.Y) + 1))
		Casing.AngularVel = RangeRand(2, 12)
		Casing.Vel = self.Vel * RangeRand(0.77, 0.97)
		MovableMan:AddParticle(Casing)
		
		Casing = CreateMOSParticle("MG Casing")
		Casing.Pos = self.Pos + self:RotateOffset(Vector(13, math.abs(self.SpriteOffset.Y) + 1))
		Casing.AngularVel = RangeRand(2, 12)
		Casing.Vel = self.Vel * RangeRand(0.77, 0.97)
		MovableMan:AddParticle(Casing)
		
		self.ammo = self.ammo - 1
	end
end
]]
function UAVDropBomb(self)
	local Bomb = CreateMOSRotating(self.bomb)
	Bomb.Pos = self.Pos + self:RotateOffset(Vector(5, math.abs(self.SpriteOffset.Y) + math.abs(Bomb.SpriteOffset.Y) - 7))
	Bomb.RotAngle = self.angle
	Bomb.Vel = self.Vel
	MovableMan:AddParticle(Bomb)
	
	self.payload = self.payload - 1
end

function UAVFindController(self)
	self.valid = false
	
	local Ctrls = {}
	for Act in MovableMan.Actors do
		if Act.Team == self.Team and not Act:IsDead() and Act.PresetName == "Turret Control" then
			table.insert(Ctrls, Act)
		end
	end
	
	if #Ctrls > 0 then
		local index = -1
		local dist = 100000
		for i = 1, #Ctrls do
			tmp = (Ctrls[i].Pos - self.Pos).Magnitude
			if tmp < dist then
				dist = tmp
				index = i
			end
		end
		
		if index > 0 then
			Ctrls[index]:AddAISceneWaypoint(Vector(self.ID,1))
			self.update = math.floor(RangeRand(1000,2000))
			return Ctrls[index]
		end
	end
	
	self.update = math.floor(RangeRand(6000,8000))
	return nil
end